from datetime import timedelta
from io import StringIO
from unittest import mock

from django.contrib.auth import get_user_model
from django.core import management
from django.db import models
from django.test import TestCase
from django.utils import timezone

from wagtail.models import Collection, Page, PageRevision, logging
from wagtail.signals import page_published, page_unpublished
from wagtail.test.testapp.models import EventPage, SimplePage


class TestFixTreeCommand(TestCase):
    fixtures = ['test.json']

    def badly_delete_page(self, page):
        # Deletes a page the wrong way.
        # This will not update numchild and may leave orphans
        models.Model.delete(page)

    def run_command(self, **options):
        options.setdefault('interactive', False)

        output = StringIO()
        management.call_command('fixtree', stdout=output, **options)
        output.seek(0)

        return output

    def test_fixes_numchild(self):
        # Get homepage and save old value
        homepage = Page.objects.get(url_path='/home/')
        old_numchild = homepage.numchild

        # Break it
        homepage.numchild = 12345
        homepage.save()

        # Check that its broken
        self.assertEqual(Page.objects.get(url_path='/home/').numchild, 12345)

        # Call command
        self.run_command()

        # Check if its fixed
        self.assertEqual(Page.objects.get(url_path='/home/').numchild, old_numchild)

    def test_fixes_depth(self):
        # Get homepage and save old value
        homepage = Page.objects.get(url_path='/home/')
        old_depth = homepage.depth

        # Break it
        homepage.depth = 12345
        homepage.save()

        # also break the root collection's depth
        root_collection = Collection.get_first_root_node()
        root_collection.depth = 42
        root_collection.save()

        # Check that its broken
        self.assertEqual(Page.objects.get(url_path='/home/').depth, 12345)
        self.assertEqual(Collection.objects.get(id=root_collection.id).depth, 42)

        # Call command
        self.run_command()

        # Check if its fixed
        self.assertEqual(Page.objects.get(url_path='/home/').depth, old_depth)
        self.assertEqual(Collection.objects.get(id=root_collection.id).depth, 1)

    def test_detects_orphans(self):
        events_index = Page.objects.get(url_path='/home/events/')
        christmas_page = EventPage.objects.get(url_path='/home/events/christmas/')

        # Delete the events index badly
        self.badly_delete_page(events_index)

        # Check that christmas_page is still in the tree
        self.assertTrue(Page.objects.filter(id=christmas_page.id).exists())

        # Call command
        output = self.run_command()

        # Check that the issues were detected
        output_string = output.read()
        self.assertIn("Incorrect numchild value found for pages: [2]", output_string)
        # Note that page ID 15 was also deleted, but is not picked up here, as
        # it is a child of 14.
        self.assertIn("Orphaned pages found: [4, 5, 6, 9, 13, 15]", output_string)

        # Check that christmas_page is still in the tree
        self.assertTrue(Page.objects.filter(id=christmas_page.id).exists())

    def test_deletes_orphans(self):
        events_index = Page.objects.get(url_path='/home/events/')
        christmas_page = EventPage.objects.get(url_path='/home/events/christmas/')

        # Delete the events index badly
        self.badly_delete_page(events_index)

        # Check that christmas_page is still in the tree
        self.assertTrue(Page.objects.filter(id=christmas_page.id).exists())

        # Call command
        # delete_orphans simulates a user pressing "y" at the prompt
        output = self.run_command(delete_orphans=True)

        # Check that the issues were detected
        output_string = output.read()
        self.assertIn("Incorrect numchild value found for pages: [2]", output_string)
        self.assertIn("7 orphaned pages deleted.", output_string)

        # Check that christmas_page has been deleted
        self.assertFalse(Page.objects.filter(id=christmas_page.id).exists())

    def test_remove_path_holes(self):
        events_index = Page.objects.get(url_path='/home/events/')
        # Delete the event page in path position 0001
        Page.objects.get(path=events_index.path + '0001').delete()

        self.run_command(full=True)
        # the gap at position 0001 should have been closed
        events_index = Page.objects.get(url_path='/home/events/')
        self.assertTrue(Page.objects.filter(path=events_index.path + '0001').exists())


class TestMovePagesCommand(TestCase):
    fixtures = ['test.json']

    def run_command(self, from_, to):
        management.call_command('move_pages', str(from_), str(to), stdout=StringIO())

    def test_move_pages(self):
        # Get pages
        events_index = Page.objects.get(url_path='/home/events/')
        about_us = Page.objects.get(url_path='/home/about-us/')
        page_ids = events_index.get_children().values_list('id', flat=True)

        # Move all events into "about us"
        self.run_command(events_index.id, about_us.id)

        # Check that all pages moved
        for page_id in page_ids:
            self.assertEqual(Page.objects.get(id=page_id).get_parent(), about_us)


class TestSetUrlPathsCommand(TestCase):

    fixtures = ['test.json']

    def run_command(self):
        management.call_command('set_url_paths', stdout=StringIO())

    def test_set_url_paths(self):
        self.run_command()


class TestReplaceTextCommand(TestCase):
    fixtures = ['test.json']

    def run_command(self, from_text, to_text):
        management.call_command('replace_text', from_text, to_text, stdout=StringIO())

    def test_replace_text(self):
        # Check that the christmas page is definitely about christmas
        christmas_page = EventPage.objects.get(url_path='/home/events/christmas/')
        self.assertEqual(christmas_page.title, "Christmas")
        self.assertEqual(christmas_page.speakers.first().last_name, "Christmas")
        self.assertEqual(christmas_page.advert_placements.first().colour, "greener than a Christmas tree")

        # Make it about easter
        self.run_command("Christmas", "Easter")

        # Check that it's now about easter
        easter_page = EventPage.objects.get(url_path='/home/events/christmas/')
        self.assertEqual(easter_page.title, "Easter")

        # Check that we also update the child objects (including advert_placements, which is defined on the superclass)
        self.assertEqual(easter_page.speakers.first().last_name, "Easter")
        self.assertEqual(easter_page.advert_placements.first().colour, "greener than a Easter tree")


class TestPublishScheduledPagesCommand(TestCase):
    def setUp(self):
        # Find root page
        self.root_page = Page.objects.get(id=2)

    def test_go_live_page_will_be_published(self):
        # Connect a mock signal handler to page_published signal
        signal_fired = [False]
        signal_page = [None]

        def page_published_handler(sender, instance, **kwargs):
            signal_fired[0] = True
            signal_page[0] = instance

        page_published.connect(page_published_handler)

        page = SimplePage(
            title="Hello world!",
            slug="hello-world",
            content="hello",
            live=False,
            has_unpublished_changes=True,
            go_live_at=timezone.now() - timedelta(days=1),
        )
        self.root_page.add_child(instance=page)

        page.save_revision(approved_go_live_at=timezone.now() - timedelta(days=1))

        p = Page.objects.get(slug='hello-world')
        self.assertFalse(p.live)
        self.assertTrue(PageRevision.objects.filter(page=p).exclude(approved_go_live_at__isnull=True).exists())

        management.call_command('publish_scheduled_pages')

        p = Page.objects.get(slug='hello-world')
        self.assertTrue(p.live)
        self.assertTrue(p.first_published_at)
        self.assertFalse(p.has_unpublished_changes)
        self.assertFalse(PageRevision.objects.filter(page=p).exclude(approved_go_live_at__isnull=True).exists())

        # Check that the page_published signal was fired
        self.assertTrue(signal_fired[0])
        self.assertEqual(signal_page[0], page)
        self.assertEqual(signal_page[0], signal_page[0].specific)

    def test_go_live_when_newer_revision_exists(self):
        page = SimplePage(
            title="Hello world!",
            slug="hello-world",
            content="hello",
            live=False,
            has_unpublished_changes=True,
            go_live_at=timezone.now() - timedelta(days=1),
        )
        self.root_page.add_child(instance=page)

        page.save_revision(approved_go_live_at=timezone.now() - timedelta(days=1))

        page.title = "Goodbye world!"
        page.save_revision(submitted_for_moderation=False)

        management.call_command('publish_scheduled_pages')

        p = Page.objects.get(slug='hello-world')
        self.assertTrue(p.live)
        self.assertTrue(p.has_unpublished_changes)
        self.assertEqual(p.title, "Hello world!")

    def test_future_go_live_page_will_not_be_published(self):
        page = SimplePage(
            title="Hello world!",
            slug="hello-world",
            content="hello",
            live=False,
            go_live_at=timezone.now() + timedelta(days=1),
        )
        self.root_page.add_child(instance=page)

        page.save_revision(approved_go_live_at=timezone.now() - timedelta(days=1))

        p = Page.objects.get(slug='hello-world')
        self.assertFalse(p.live)
        self.assertTrue(PageRevision.objects.filter(page=p).exclude(approved_go_live_at__isnull=True).exists())

        management.call_command('publish_scheduled_pages')

        p = Page.objects.get(slug='hello-world')
        self.assertFalse(p.live)
        self.assertTrue(PageRevision.objects.filter(page=p).exclude(approved_go_live_at__isnull=True).exists())

    def test_expired_page_will_be_unpublished(self):
        # Connect a mock signal handler to page_unpublished signal
        signal_fired = [False]
        signal_page = [None]

        def page_unpublished_handler(sender, instance, **kwargs):
            signal_fired[0] = True
            signal_page[0] = instance

        page_unpublished.connect(page_unpublished_handler)

        page = SimplePage(
            title="Hello world!",
            slug="hello-world",
            content="hello",
            live=True,
            has_unpublished_changes=False,
            expire_at=timezone.now() - timedelta(days=1),
        )
        self.root_page.add_child(instance=page)

        p = Page.objects.get(slug='hello-world')
        self.assertTrue(p.live)

        management.call_command('publish_scheduled_pages')

        p = Page.objects.get(slug='hello-world')
        self.assertFalse(p.live)
        self.assertTrue(p.has_unpublished_changes)
        self.assertTrue(p.expired)

        # Check that the page_published signal was fired
        self.assertTrue(signal_fired[0])
        self.assertEqual(signal_page[0], page)
        self.assertEqual(signal_page[0], signal_page[0].specific)

    def test_future_expired_page_will_not_be_unpublished(self):
        page = SimplePage(
            title="Hello world!",
            slug="hello-world",
            content="hello",
            live=True,
            expire_at=timezone.now() + timedelta(days=1),
        )
        self.root_page.add_child(instance=page)

        p = Page.objects.get(slug='hello-world')
        self.assertTrue(p.live)

        management.call_command('publish_scheduled_pages')

        p = Page.objects.get(slug='hello-world')
        self.assertTrue(p.live)
        self.assertFalse(p.expired)

    def test_expired_pages_are_dropped_from_mod_queue(self):
        page = SimplePage(
            title="Hello world!",
            slug="hello-world",
            content="hello",
            live=False,
            expire_at=timezone.now() - timedelta(days=1),
        )
        self.root_page.add_child(instance=page)

        page.save_revision(submitted_for_moderation=True)

        p = Page.objects.get(slug='hello-world')
        self.assertFalse(p.live)
        self.assertTrue(PageRevision.objects.filter(page=p, submitted_for_moderation=True).exists())

        management.call_command('publish_scheduled_pages')

        p = Page.objects.get(slug='hello-world')
        self.assertFalse(PageRevision.objects.filter(page=p, submitted_for_moderation=True).exists())


class TestPurgeRevisionsCommand(TestCase):
    fixtures = ['test.json']

    def setUp(self):
        # Find root page
        self.root_page = Page.objects.get(id=2)
        self.page = SimplePage(
            title="Hello world!",
            slug="hello-world",
            content="hello",
            live=False,
        )
        self.root_page.add_child(instance=self.page)
        self.page.refresh_from_db()

    def run_command(self, days=None):
        if days:
            days_input = '--days=' + str(days)
            return management.call_command('purge_revisions', days_input, stdout=StringIO())
        return management.call_command('purge_revisions', stdout=StringIO())

    def test_latest_revision_not_purged(self):

        revision_1 = self.page.save_revision()

        revision_2 = self.page.save_revision()

        self.run_command()

        # revision 1 should be deleted, revision 2 should not be
        self.assertNotIn(revision_1, PageRevision.objects.filter(page=self.page))
        self.assertIn(revision_2, PageRevision.objects.filter(page=self.page))

    def test_revisions_in_moderation_not_purged(self):

        self.page.save_revision(submitted_for_moderation=True)

        revision = self.page.save_revision()

        self.run_command()

        self.assertTrue(PageRevision.objects.filter(page=self.page, submitted_for_moderation=True).exists())

        try:
            from wagtail.models import Task, Workflow, WorkflowTask
            workflow = Workflow.objects.create(name='test_workflow')
            task_1 = Task.objects.create(name='test_task_1')
            user = get_user_model().objects.first()
            WorkflowTask.objects.create(workflow=workflow, task=task_1, sort_order=1)
            workflow.start(self.page, user)
            self.page.save_revision()
            self.run_command()
            # even though no longer the latest revision, the old revision should stay as it is
            # attached to an in progress workflow
            self.assertIn(revision, PageRevision.objects.filter(page=self.page))
        except ImportError:
            pass

    def test_revisions_with_approve_go_live_not_purged(self):

        approved_revision = self.page.save_revision(approved_go_live_at=timezone.now() + timedelta(days=1))

        self.page.save_revision()

        self.run_command()

        self.assertIn(approved_revision, PageRevision.objects.filter(page=self.page))

    def test_purge_revisions_with_date_cutoff(self):

        old_revision = self.page.save_revision()

        self.page.save_revision()

        self.run_command(days=30)

        # revision should not be deleted, as it is younger than 30 days
        self.assertIn(old_revision, PageRevision.objects.filter(page=self.page))

        old_revision.created_at = timezone.now() - timedelta(days=31)
        old_revision.save()

        self.run_command(days=30)

        # revision is now older than 30 days, so should be deleted
        self.assertNotIn(old_revision, PageRevision.objects.filter(page=self.page))


class TestCreateLogEntriesFromRevisionsCommand(TestCase):
    fixtures = ['test.json']

    def setUp(self):
        self.page = SimplePage(
            title="Hello world!",
            slug="hello-world",
            content="hello",
            live=False,
            expire_at=timezone.now() - timedelta(days=1),
        )

        Page.objects.get(id=2).add_child(instance=self.page)

        # Create empty revisions, which should not be converted to log entries
        for i in range(3):
            self.page.save_revision()

        # Add another revision with a content change
        self.page.title = "Hello world!!"
        revision = self.page.save_revision()
        revision.publish()

        # clean up log entries
        logging.PageLogEntry.objects.all().delete()

    def test_log_entries_created_from_revisions(self):
        management.call_command('create_log_entries_from_revisions')

        # Should not create entries for empty revisions.
        self.assertListEqual(
            list(logging.PageLogEntry.objects.values_list("action", flat=True)),
            ['wagtail.publish', 'wagtail.edit', 'wagtail.create']
        )

    def test_command_doesnt_crash_for_revisions_without_page_model(self):
        with mock.patch(
            'wagtail.models.pages.ContentType.model_class',
            return_value=None,
        ):
            management.call_command('create_log_entries_from_revisions')
            self.assertEqual(logging.PageLogEntry.objects.count(), 0)



# Generated by Django 2.1.7 on 2019-03-28 02:30

from django.db import migrations, models
import wagtail.blocks
import wagtail.fields
import wagtail.contrib.images.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0057_customdocumentwithauthor'),
    ]

    operations = [
        migrations.CreateModel(
            name='BlockCountsStreamModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('body', wagtail.fields.StreamField([('text', wagtail.blocks.CharBlock()), ('rich_text', wagtail.blocks.RichTextBlock()), ('image', wagtail.contrib.images.blocks.ImageChooserBlock())])),
            ],
        ),
        migrations.CreateModel(
            name='MinMaxCountStreamModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('body', wagtail.fields.StreamField([('text', wagtail.blocks.CharBlock()), ('rich_text', wagtail.blocks.RichTextBlock()), ('image', wagtail.contrib.images.blocks.ImageChooserBlock())])),
            ],
        ),
    ]



from django.urls import path

from wagtail.contrib.images.views import chooser, images, multiple


app_name = 'wagtailimages'
urlpatterns = [
    path('', images.IndexView.as_view(), name='index'),
    path('results/', images.ListingResultsView.as_view(), name='listing_results'),
    path('<int:image_id>/', images.edit, name='edit'),
    path('<int:image_id>/delete/', images.delete, name='delete'),
    path('<int:image_id>/generate_url/', images.url_generator, name='url_generator'),
    path('<int:image_id>/generate_url/<str:filter_spec>/', images.generate_url, name='generate_url'),
    path('<int:image_id>/preview/<str:filter_spec>/', images.preview, name='preview'),
    path('add/', images.add, name='add'),
    path('usage/<int:image_id>/', images.usage, name='image_usage'),

    path('multiple/add/', multiple.AddView.as_view(), name='add_multiple'),
    path('multiple/<int:image_id>/', multiple.EditView.as_view(), name='edit_multiple'),
    path('multiple/create_from_uploaded_image/<int:uploaded_image_id>/', multiple.CreateFromUploadedImageView.as_view(), name='create_multiple_from_uploaded_image'),
    path('multiple/<int:image_id>/delete/', multiple.DeleteView.as_view(), name='delete_multiple'),
    path('multiple/delete_upload/<int:uploaded_image_id>/', multiple.DeleteUploadView.as_view(), name='delete_upload_multiple'),

    path('chooser/', chooser.ChooseView.as_view(), name='chooser'),
    path('chooser/results/', chooser.ChooseResultsView.as_view(), name='chooser_results'),
    path('chooser/<int:image_id>/', chooser.image_chosen, name='image_chosen'),
    path('chooser/upload/', chooser.chooser_upload, name='chooser_upload'),
    path('chooser/<int:image_id>/select_format/', chooser.chooser_select_format, name='chooser_select_format'),
]



import pytz

from django.conf import settings
from django.utils.dates import MONTHS, WEEKDAYS, WEEKDAYS_ABBR
from django.utils.translation import gettext as _


# Wagtail languages with >=90% coverage
# This list is manually maintained
WAGTAILADMIN_PROVIDED_LANGUAGES = [
    ('ar', 'Arabic'),
    ('ca', 'Catalan'),
    ('cs', 'Czech'),
    ('de', 'German'),
    ('el', 'Greek'),
    ('en', 'English'),
    ('es', 'Spanish'),
    ('et', 'Estonian'),
    ('fi', 'Finnish'),
    ('fr', 'French'),
    ('gl', 'Galician'),
    ('hr', 'Croatian'),
    ('hu', 'Hungarian'),
    ('id-id', 'Indonesian'),
    ('is-is', 'Icelandic'),
    ('it', 'Italian'),
    ('ja', 'Japanese'),
    ('ko', 'Korean'),
    ('lt', 'Lithuanian'),
    ('mn', 'Mongolian'),
    ('nb', 'Norwegian Bokmål'),
    ('nl', 'Dutch'),
    ('fa', 'Persian'),
    ('pl', 'Polish'),
    ('pt-br', 'Brazilian Portuguese'),
    ('pt-pt', 'Portuguese'),
    ('ro', 'Romanian'),
    ('ru', 'Russian'),
    ('sv', 'Swedish'),
    ('sk-sk', 'Slovak'),
    ('th', 'Thai'),
    ('tr', 'Turkish'),
    ('uk', 'Ukrainian'),
    ('zh-hans', 'Chinese (Simplified)'),
    ('zh-hant', 'Chinese (Traditional)'),
]


# Translatable strings to be made available to JavaScript code
# as the wagtailConfig.STRINGS object
def get_js_translation_strings():
    return {
        'DELETE': _('Delete'),
        'EDIT': _('Edit'),
        'PAGE': _('Page'),
        'PAGES': _('Pages'),
        'LOADING': _('Loading…'),
        'NO_RESULTS': _('No results'),
        'SERVER_ERROR': _('Server Error'),
        'SEE_ALL': _('See all'),
        'CLOSE_EXPLORER': _('Close explorer'),
        'ALT_TEXT': _('Alt text'),
        'DECORATIVE_IMAGE': _('Decorative image'),
        'WRITE_HERE': _('Write here…'),
        'HORIZONTAL_LINE': _('Horizontal line'),
        'LINE_BREAK': _('Line break'),
        'UNDO': _('Undo'),
        'REDO': _('Redo'),
        'RELOAD_PAGE': _('Reload the page'),
        'RELOAD_EDITOR': _('Reload saved content'),
        'SHOW_LATEST_CONTENT': _('Show latest content'),
        'SHOW_ERROR': _('Show error'),
        'EDITOR_CRASH': _('The editor just crashed. Content has been reset to the last saved version.'),
        'BROKEN_LINK': _('Broken link'),
        'MISSING_DOCUMENT': _('Missing document'),
        'CLOSE': _('Close'),
        'EDIT_PAGE': _('Edit \'{title}\''),
        'VIEW_CHILD_PAGES_OF_PAGE': _('View child pages of \'{title}\''),
        'PAGE_EXPLORER': _('Page explorer'),
        'SAVE': _('Save'),
        'SAVING': _('Saving...'),
        'CANCEL': _('Cancel'),
        'DELETING': _('Deleting...'),
        'ADD_A_COMMENT': _('Add a comment'),
        'SHOW_COMMENTS': _('Show comments'),
        'REPLY': _('Reply'),
        'RESOLVE': _('Resolve'),
        'RETRY': _('Retry'),
        'DELETE_ERROR': _('Delete error'),
        'CONFIRM_DELETE_COMMENT': _('Are you sure?'),
        'SAVE_ERROR': _('Save error'),
        'SAVE_COMMENT_WARNING': _('This will be saved when the page is saved'),
        'FOCUS_COMMENT': _('Focus comment'),
        'UNFOCUS_COMMENT': _('Unfocus comment'),
        'COMMENT': _('Comment'),
        'MORE_ACTIONS': _('More actions'),
        'SAVE_PAGE_TO_ADD_COMMENT': _('Save the page to add this comment'),
        'SAVE_PAGE_TO_SAVE_COMMENT_CHANGES': _('Save the page to save this comment'),
        'SAVE_PAGE_TO_SAVE_REPLY': _('Save the page to save this reply'),
        'DASHBOARD': _('Dashboard'),
        'EDIT_YOUR_ACCOUNT': _('Edit your account'),
        'SEARCH': _('Search'),

        'MONTHS': [str(m) for m in MONTHS.values()],

        # Django's WEEKDAYS list begins on Monday, but ours should start on Sunday, so start
        # counting from -1 and use modulo 7 to get an array index
        'WEEKDAYS': [str(WEEKDAYS[d % 7]) for d in range(-1, 6)],
        'WEEKDAYS_SHORT': [str(WEEKDAYS_ABBR[d % 7]) for d in range(-1, 6)],

        # used by bulk actions
        'BULK_ACTIONS': {
            'PAGE': {
                'SINGULAR': _('1 page selected'),
                'PLURAL': _("{0} pages selected"),
                'ALL': _("All {0} pages on this screen selected"),
                'ALL_IN_LISTING': _("All pages in listing selected"),
            },
            'DOCUMENT': {
                'SINGULAR': _('1 document selected'),
                'PLURAL': _("{0} documents selected"),
                'ALL': _("All {0} documents on this screen selected"),
                'ALL_IN_LISTING': _("All documents in listing selected"),
            },
            'IMAGE': {
                'SINGULAR': _('1 image selected'),
                'PLURAL': _("{0} images selected"),
                'ALL': _("All {0} images on this screen selected"),
                'ALL_IN_LISTING': _("All images in listing selected"),
            },
            'USER': {
                'SINGULAR': _('1 user selected'),
                'PLURAL': _("{0} users selected"),
                'ALL': _("All {0} users on this screen selected"),
                'ALL_IN_LISTING': _("All users in listing selected"),
            },
            'ITEM': {
                'SINGULAR': _('1 item selected'),
                'PLURAL': _("{0} items selected"),
                'ALL': _("All {0} items on this screen selected"),
                'ALL_IN_LISTING': _("All items in listing selected"),
            },
        },
    }


def get_available_admin_languages():
    return getattr(settings, 'WAGTAILADMIN_PERMITTED_LANGUAGES', WAGTAILADMIN_PROVIDED_LANGUAGES)


def get_available_admin_time_zones():
    if not settings.USE_TZ:
        return []

    return getattr(settings, 'WAGTAIL_USER_TIME_ZONES', pytz.common_timezones)



from django import forms

from wagtail.admin.views.bulk_action import BulkAction
from wagtail.admin.views.pages.search import page_filter_search
from wagtail.models import Page


class DefaultPageForm(forms.Form):
    include_descendants = forms.BooleanField(required=False)


class PageBulkAction(BulkAction):
    models = [Page]
    form_class = DefaultPageForm

    def get_all_objects_in_listing_query(self, parent_id):
        listing_objects = self.model.objects.all()

        if parent_id is not None:
            listing_objects = listing_objects.get(id=parent_id).get_children()

        listing_objects = listing_objects.values_list('pk', flat=True)

        if 'q' in self.request.GET:
            q = self.request.GET.get('q', '')

            listing_objects = page_filter_search(q, listing_objects)[0].results()

        return listing_objects

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['items_with_no_access'] = [
            {'item': page, 'can_edit': page.permissions_for_user(self.request.user).can_edit()} for page in context['items_with_no_access']
        ]
        return context

    def get_execution_context(self):
        return {
            'user': self.request.user
        }



from django.contrib.auth.models import Group, Permission
from django.db import models
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from treebeard.mp_tree import MP_Node

from wagtail.models.query import TreeQuerySet
from wagtail.search import index
from wagtail.utils.treebeard import TreebeardPathFixMixin

from .view_restrictions import BaseViewRestriction


class BaseCollectionManager(models.Manager):
    def get_queryset(self):
        return TreeQuerySet(self.model).order_by('path')


CollectionManager = BaseCollectionManager.from_queryset(TreeQuerySet)


class CollectionViewRestriction(BaseViewRestriction):
    collection = models.ForeignKey(
        'Collection',
        verbose_name=_('collection'),
        related_name='view_restrictions',
        on_delete=models.CASCADE
    )

    passed_view_restrictions_session_key = 'passed_collection_view_restrictions'

    class Meta:
        verbose_name = _('collection view restriction')
        verbose_name_plural = _('collection view restrictions')


class Collection(TreebeardPathFixMixin, MP_Node):
    """
    A location in which resources such as images and documents can be grouped
    """
    name = models.CharField(max_length=255, verbose_name=_('name'))

    objects = CollectionManager()
    # Tell treebeard to order Collections' paths such that they are ordered by name at each level.
    node_order_by = ['name']

    def __str__(self):
        return self.name

    def get_ancestors(self, inclusive=False):
        return Collection.objects.ancestor_of(self, inclusive)

    def get_descendants(self, inclusive=False):
        return Collection.objects.descendant_of(self, inclusive)

    def get_siblings(self, inclusive=True):
        return Collection.objects.sibling_of(self, inclusive)

    def get_next_siblings(self, inclusive=False):
        return self.get_siblings(inclusive).filter(path__gte=self.path).order_by('path')

    def get_prev_siblings(self, inclusive=False):
        return self.get_siblings(inclusive).filter(path__lte=self.path).order_by('-path')

    def get_view_restrictions(self):
        """Return a query set of all collection view restrictions that apply to this collection"""
        return CollectionViewRestriction.objects.filter(collection__in=self.get_ancestors(inclusive=True))

    def get_indented_name(self, indentation_start_depth=2, html=False):
        """
        Renders this Collection's name as a formatted string that displays its hierarchical depth via indentation.
        If indentation_start_depth is supplied, the Collection's depth is rendered relative to that depth.
        indentation_start_depth defaults to 2, the depth of the first non-Root Collection.
        Pass html=True to get a HTML representation, instead of the default plain-text.

        Example text output: "    ↳ Pies"
        Example HTML output: "&nbsp;&nbsp;&nbsp;&nbsp;&#x21b3 Pies"
        """
        display_depth = self.depth - indentation_start_depth
        # A Collection with a display depth of 0 or less (Root's can be -1), should have no indent.
        if display_depth <= 0:
            return self.name

        # Indent each level of depth by 4 spaces (the width of the ↳ character in our admin font), then add ↳
        # before adding the name.
        if html:
            # NOTE: &#x21b3 is the hex HTML entity for ↳.
            return format_html(
                "{indent}{icon} {name}",
                indent=mark_safe('&nbsp;' * 4 * display_depth),
                icon=mark_safe('&#x21b3'),
                name=self.name
            )
        # Output unicode plain-text version
        return "{}↳ {}".format(' ' * 4 * display_depth, self.name)

    class Meta:
        verbose_name = _('collection')
        verbose_name_plural = _('collections')


def get_root_collection_id():
    return Collection.get_first_root_node().id


class CollectionMember(models.Model):
    """
    Base class for models that are categorised into collections
    """
    collection = models.ForeignKey(
        Collection,
        default=get_root_collection_id,
        verbose_name=_('collection'),
        related_name='+',
        on_delete=models.CASCADE
    )

    search_fields = [
        index.FilterField('collection'),
    ]

    class Meta:
        abstract = True


class GroupCollectionPermissionManager(models.Manager):
    def get_by_natural_key(self, group, collection, permission):
        return self.get(group=group,
                        collection=collection,
                        permission=permission)


class GroupCollectionPermission(models.Model):
    """
    A rule indicating that a group has permission for some action (e.g. "create document")
    within a specified collection.
    """
    group = models.ForeignKey(
        Group,
        verbose_name=_('group'),
        related_name='collection_permissions',
        on_delete=models.CASCADE
    )
    collection = models.ForeignKey(
        Collection,
        verbose_name=_('collection'),
        related_name='group_permissions',
        on_delete=models.CASCADE
    )
    permission = models.ForeignKey(
        Permission,
        verbose_name=_('permission'),
        on_delete=models.CASCADE
    )

    def __str__(self):
        return "Group %d ('%s') has permission '%s' on collection %d ('%s')" % (
            self.group.id, self.group,
            self.permission,
            self.collection.id, self.collection
        )

    def natural_key(self):
        return (self.group, self.collection, self.permission)

    objects = GroupCollectionPermissionManager()

    class Meta:
        unique_together = ('group', 'collection', 'permission')
        verbose_name = _('group collection permission')
        verbose_name_plural = _('group collection permissions')



from django import forms
from django.utils.translation import gettext_lazy as _


class ButtonSelect(forms.Select):
    """
    A select widget for fields with choices. Displays as a list of buttons.
    """
    input_type = 'hidden'
    template_name = 'wagtailadmin/widgets/button_select.html'
    option_template_name = 'wagtailadmin/widgets/button_select_option.html'


class BooleanButtonSelect(ButtonSelect):
    """
    A select widget for boolean fields. Displays as three buttons. "All", "Yes" and "No".
    """
    def __init__(self, attrs=None):
        choices = (
            ('', _("All")),
            ('true', _("Yes")),
            ('false', _("No")),
        )
        super().__init__(attrs, choices)

    def format_value(self, value):
        try:
            return {
                True: ['true'], False: ['false'],
                'true': ['true'], 'false': ['false'],
            }[value]
        except KeyError:
            return ''

    def value_from_datadict(self, data, files, name):
        value = data.get(name)
        return {
            True: True,
            'True': True,
            'False': False,
            False: False,
            'true': True,
            'false': False,
        }.get(value)



# Generated by Django 3.0.10 on 2020-12-19 15:07

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailadmin', '0001_create_admin_access_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='Admin',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': [('access_admin', 'Can access Wagtail admin')],
                'managed': False,
                'default_permissions': [],
            },
        ),
    ]



from django.contrib.admin.utils import quote
from django.core import checks
from django.urls import reverse

from wagtail.admin.admin_url_finder import register_admin_url_finder
from wagtail.admin.checks import check_panels_in_model
from wagtail.models.admin import get_object_usage


SNIPPET_MODELS = []


def get_snippet_models():
    return SNIPPET_MODELS


class SnippetAdminURLFinder:
    # subclasses should define a 'model' attribute
    def __init__(self, user=None):
        if user:
            from wagtail.contrib.snippets.permissions import get_permission_name
            self.user_can_edit = user.has_perm(get_permission_name('change', self.model))
        else:
            # skip permission checks
            self.user_can_edit = True

    def get_edit_url(self, instance):
        if self.user_can_edit:
            return reverse('wagtailsnippets:edit', args=(
                self.model._meta.app_label, self.model._meta.model_name, quote(instance.pk)
            ))


def register_snippet(model):
    if model not in SNIPPET_MODELS:
        model.get_usage = get_object_usage
        model.usage_url = get_snippet_usage_url
        SNIPPET_MODELS.append(model)
        SNIPPET_MODELS.sort(key=lambda x: x._meta.verbose_name)

        url_finder_class = type('_SnippetAdminURLFinder', (SnippetAdminURLFinder, ), {'model': model})
        register_admin_url_finder(model, url_finder_class)

        @checks.register('panels')
        def modeladmin_model_check(app_configs, **kwargs):
            errors = check_panels_in_model(model, 'snippets')
            return errors

    return model


def get_snippet_usage_url(self):
    return reverse('wagtailsnippets:usage', args=(
        self._meta.app_label, self._meta.model_name, quote(self.pk)))



import datetime

from io import BytesIO

from django.conf import settings
from django.conf.locale import LANG_INFO
from django.test import TestCase
from django.test.utils import override_settings
from django.urls import reverse
from django.utils import timezone, translation
from openpyxl import load_workbook

from wagtail.admin.views.mixins import ExcelDateFormatter
from wagtail.models import Page, logging
from wagtail.test.utils import WagtailTestUtils


class TestLockedPagesView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.user = self.login()

    def get(self, params={}):
        return self.client.get(reverse('wagtailadmin_reports:locked_pages'), params)

    def test_simple(self):
        response = self.get()
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailadmin/reports/locked_pages.html')

        # Initially there should be no locked pages
        self.assertContains(response, "No locked pages found.")

        self.page = Page.objects.first()
        self.page.locked = True
        self.page.locked_by = self.user
        self.page.locked_at = timezone.now()
        self.page.save()

        # Now the listing should contain our locked page
        response = self.get()
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'wagtailadmin/reports/locked_pages.html')
        self.assertNotContains(response, "No locked pages found.")
        self.assertContains(response, self.page.title)

    def test_csv_export(self):

        self.page = Page.objects.first()
        self.page.locked = True
        self.page.locked_by = self.user
        if settings.USE_TZ:
            # 12:00 UTC
            self.page.locked_at = '2013-02-01T12:00:00.000Z'
            self.page.latest_revision_created_at = '2013-01-01T12:00:00.000Z'
        else:
            # 12:00 in no specific timezone
            self.page.locked_at = '2013-02-01T12:00:00'
            self.page.latest_revision_created_at = '2013-01-01T12:00:00'
        self.page.save()

        response = self.get(params={'export': 'csv'})

        # Check response
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")
        self.assertEqual(data_lines[0], 'Title,Updated,Status,Type,Locked At,Locked By\r')
        if settings.USE_TZ:
            self.assertEqual(data_lines[1], 'Root,2013-01-01 12:00:00+00:00,live,Page,2013-02-01 12:00:00+00:00,test@email.com\r')
        else:
            self.assertEqual(data_lines[1], 'Root,2013-01-01 12:00:00,live,Page,2013-02-01 12:00:00,test@email.com\r')

    def test_xlsx_export(self):

        self.page = Page.objects.first()
        self.page.locked = True
        self.page.locked_by = self.user
        if settings.USE_TZ:
            # 12:00 UTC
            self.page.locked_at = '2013-02-01T12:00:00.000Z'
            self.page.latest_revision_created_at = '2013-01-01T12:00:00.000Z'
        else:
            # 12:00 in no specific timezone
            self.page.locked_at = '2013-02-01T12:00:00'
            self.page.latest_revision_created_at = '2013-01-01T12:00:00'
        self.page.save()

        response = self.get(params={'export': 'xlsx'})

        # Check response - the locked page info should be in it
        self.assertEqual(response.status_code, 200)
        workbook_data = response.getvalue()
        worksheet = load_workbook(filename=BytesIO(workbook_data))['Sheet1']
        cell_array = [[cell.value for cell in row] for row in worksheet.rows]
        self.assertEqual(cell_array[0], ['Title', 'Updated', 'Status', 'Type', 'Locked At', 'Locked By'])
        self.assertEqual(cell_array[1], ['Root', datetime.datetime(2013, 1, 1, 12, 0), 'live', 'Page', datetime.datetime(2013, 2, 1, 12, 0), 'test@email.com'])
        self.assertEqual(len(cell_array), 2)


class TestFilteredLockedPagesView(TestCase, WagtailTestUtils):
    fixtures = ['test.json']

    def setUp(self):
        self.user = self.login()
        self.unpublished_page = Page.objects.get(url_path='/home/events/tentative-unpublished-event/')
        self.unpublished_page.locked = True
        self.unpublished_page.locked_by = self.user
        self.unpublished_page.locked_at = timezone.now()
        self.unpublished_page.save()

    def get(self, params={}):
        return self.client.get(reverse('wagtailadmin_reports:locked_pages'), params)

    def test_filter_by_live(self):
        response = self.get(params={'live': 'true'})
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, "Tentative Unpublished Event")
        self.assertContains(response, "My locked page")

        response = self.get(params={'live': 'false'})
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Tentative Unpublished Event")
        self.assertNotContains(response, "My locked page")

    def test_filter_by_user(self):
        response = self.get(params={'locked_by': self.user.pk})
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Tentative Unpublished Event")
        self.assertNotContains(response, "My locked page")


class TestFilteredLogEntriesView(TestCase, WagtailTestUtils):
    fixtures = ['test.json']

    def setUp(self):
        self.user = self.login()
        self.home_page = Page.objects.get(url_path='/home/')

        self.create_log = logging.PageLogEntry.objects.log_action(self.home_page, 'wagtail.create')
        self.edit_log_1 = logging.PageLogEntry.objects.log_action(self.home_page, 'wagtail.edit')
        self.edit_log_2 = logging.PageLogEntry.objects.log_action(self.home_page, 'wagtail.edit')
        self.edit_log_3 = logging.PageLogEntry.objects.log_action(self.home_page, 'wagtail.edit')

        self.create_comment_log = logging.PageLogEntry.objects.log_action(self.home_page, 'wagtail.comments.create', data={
            'comment': {
                'contentpath': 'title',
                'text': 'Foo',
            }
        })
        self.edit_comment_log = logging.PageLogEntry.objects.log_action(self.home_page, 'wagtail.comments.edit', data={
            'comment': {
                'contentpath': 'title',
                'text': 'Edited',
            }
        })
        self.create_reply_log = logging.PageLogEntry.objects.log_action(self.home_page, 'wagtail.comments.create_reply', data={
            'comment': {
                'contentpath': 'title',
                'text': 'Foo',
            }
        })

    def get(self, params={}):
        return self.client.get(reverse('wagtailadmin_reports:site_history'), params)

    def assert_log_entries(self, response, expected):
        actual = set(response.context['object_list'])
        self.assertSetEqual(actual, set(expected))

    def test_unfiltered(self):
        response = self.get()
        self.assertEqual(response.status_code, 200)
        self.assert_log_entries(response, [
            self.create_log,
            self.edit_log_1,
            self.edit_log_2,
            self.edit_log_3,
            self.create_comment_log,
            self.edit_comment_log,
            self.create_reply_log,
        ])

    def test_filter_by_action(self):
        response = self.get(params={'action': 'wagtail.edit'})
        self.assertEqual(response.status_code, 200)
        self.assert_log_entries(response, [
            self.edit_log_1,
            self.edit_log_2,
            self.edit_log_3,
        ])

    def test_hide_commenting_actions(self):
        response = self.get(params={'hide_commenting_actions': 'on'})
        self.assertEqual(response.status_code, 200)
        self.assert_log_entries(response, [
            self.create_log,
            self.edit_log_1,
            self.edit_log_2,
            self.edit_log_3,
        ])


@override_settings(
    USE_L10N=True,
)
class TestExcelDateFormatter(TestCase):
    def test_all_locales(self):
        formatter = ExcelDateFormatter()

        for lang in LANG_INFO.keys():
            with self.subTest(lang), translation.override(lang):
                self.assertNotEqual(formatter.get(), "")



from django.http import HttpResponseForbidden
from django.shortcuts import get_object_or_404, redirect
from django.utils.translation import gettext_lazy

from wagtail import hooks
from wagtail.admin import messages
from wagtail.admin.forms.collections import CollectionForm
from wagtail.admin.views.generic import CreateView, DeleteView, EditView, IndexView
from wagtail.models import Collection, GroupCollectionPermission
from wagtail.permissions import collection_permission_policy


class Index(IndexView):
    permission_policy = collection_permission_policy
    model = Collection
    context_object_name = 'collections'
    template_name = 'wagtailadmin/collections/index.html'
    add_url_name = 'wagtailadmin_collections:add'
    index_url_name = 'wagtailadmin_collections:index'
    page_title = gettext_lazy("Collections")
    add_item_label = gettext_lazy("Add a collection")
    header_icon = 'folder-open-1'

    def get_queryset(self):
        return self.permission_policy.instances_user_has_any_permission_for(
            self.request.user, ['add', 'change', 'delete']
        ).exclude(depth=1)


class Create(CreateView):
    permission_policy = collection_permission_policy
    form_class = CollectionForm
    page_title = gettext_lazy("Add collection")
    success_message = gettext_lazy("Collection '{0}' created.")
    add_url_name = 'wagtailadmin_collections:add'
    edit_url_name = 'wagtailadmin_collections:edit'
    index_url_name = 'wagtailadmin_collections:index'
    header_icon = 'folder-open-1'

    def get_form(self, form_class=None):
        form = super().get_form(form_class)
        # Now filter collections offered in parent field by current user's add permissions
        collections = self.permission_policy.instances_user_has_permission_for(self.request.user, 'add')
        form.fields['parent'].queryset = collections
        return form

    def save_instance(self):
        instance = self.form.save(commit=False)
        parent = self.form.cleaned_data['parent']
        parent.add_child(instance=instance)
        return instance


class Edit(EditView):
    permission_policy = collection_permission_policy
    model = Collection
    form_class = CollectionForm
    template_name = 'wagtailadmin/collections/edit.html'
    success_message = gettext_lazy("Collection '{0}' updated.")
    error_message = gettext_lazy("The collection could not be saved due to errors.")
    delete_item_label = gettext_lazy("Delete collection")
    edit_url_name = 'wagtailadmin_collections:edit'
    index_url_name = 'wagtailadmin_collections:index'
    delete_url_name = 'wagtailadmin_collections:delete'
    context_object_name = 'collection'
    header_icon = 'folder-open-1'

    def _user_may_move_collection(self, user, instance):
        """
        Is this instance used for assigning GroupCollectionPermissions to the user?
        If so, this user is not allowed do move the collection to a new part of the tree
        """
        if user.is_active and user.is_superuser:
            return True
        else:
            permissions = self.permission_policy._get_permission_objects_for_actions(['add', 'edit', 'delete'])
            return not GroupCollectionPermission.objects.filter(
                group__user=user,
                permission__in=permissions,
                collection=instance,
            ).exists()

    def get_queryset(self):
        return self.permission_policy.instances_user_has_permission_for(
            self.request.user, 'change'
        ).exclude(depth=1)

    def get_form(self, form_class=None):
        form = super().get_form(form_class)
        user = self.request.user
        # if user does not have add permission anywhere, they can't move a collection
        if not self.permission_policy.user_has_permission(user, 'add'):
            form.fields.pop('parent')
        # If this instance is a collection used to assign permissions for this user,
        # do not let the user move this collection.
        elif not self._user_may_move_collection(user, form.instance):
            form.fields.pop('parent')
        else:
            # Filter collections offered in parent field by current user's add permissions
            collections = self.permission_policy.instances_user_has_permission_for(user, 'add')
            form.fields['parent'].queryset = collections
            # Disable unavailable options in CollectionChoiceField select widget
            form.fields['parent'].disabled_queryset = form.instance.get_descendants(inclusive=True)

        form.initial['parent'] = form.instance.get_parent().pk
        return form

    def save_instance(self):
        instance = self.form.save()
        if 'parent' in self.form.changed_data:
            instance.move(self.form.cleaned_data['parent'], 'sorted-child')
        return instance

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['can_delete'] = self.permission_policy.instances_user_has_permission_for(
            self.request.user, 'delete'
        ).filter(pk=self.object.pk).first()
        return context


class Delete(DeleteView):
    permission_policy = collection_permission_policy
    model = Collection
    success_message = gettext_lazy("Collection '{0}' deleted.")
    index_url_name = 'wagtailadmin_collections:index'
    delete_url_name = 'wagtailadmin_collections:delete'
    page_title = gettext_lazy("Delete collection")
    confirmation_message = gettext_lazy("Are you sure you want to delete this collection?")
    header_icon = 'folder-open-1'

    def get_queryset(self):
        return self.permission_policy.instances_user_has_permission_for(
            self.request.user, 'delete'
        ).exclude(depth=1)

    def get_collection_contents(self):
        collection_contents = [
            hook(self.object)
            for hook in hooks.get_hooks('describe_collection_contents')
        ]

        # filter out any hook responses that report that the collection is empty
        # (by returning None, or a dict with 'count': 0)
        def is_nonempty(item_type):
            return item_type and item_type['count'] > 0

        return list(filter(is_nonempty, collection_contents))

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        collection_contents = self.get_collection_contents()

        if collection_contents:
            # collection is non-empty; render the 'not allowed to delete' response
            self.template_name = 'wagtailadmin/collections/delete_not_empty.html'
            context['collection_contents'] = collection_contents

        return context

    def post(self, request, pk):
        self.object = get_object_or_404(self.get_queryset(), id=pk)
        collection_contents = self.get_collection_contents()

        if collection_contents:
            # collection is non-empty; refuse to delete it
            return HttpResponseForbidden()

        self.object.delete()
        messages.success(request, self.success_message.format(self.object))
        return redirect(self.index_url_name)



from django.urls import path

from wagtail.test.testapp.views import bob_only_zone, message_test


urlpatterns = [
    path('bob-only-zone', bob_only_zone, name='testapp_bob_only_zone'),
    path('messages/', message_test, name='testapp_message_test')
]



from django.urls import path

from wagtail.contrib.redirects import views


app_name = 'wagtailredirects'
urlpatterns = [
    path('', views.index, name='index'),
    path('add/', views.add, name='add'),
    path('<int:redirect_id>/', views.edit, name='edit'),
    path('<int:redirect_id>/delete/', views.delete, name='delete'),
    path('import/', views.start_import, name="start_import"),
    path('import/process/', views.process_import, name="process_import"),
    path('report', views.RedirectsReportView.as_view(), name="report"),
]



from django.urls import path, re_path

from wagtail.admin.views import page_privacy
from wagtail.admin.views.pages import (
    convert_alias, copy, create, delete, edit, history, lock, moderation, move, ordering, preview,
    revisions, search, unpublish, usage, workflow)


app_name = 'wagtailadmin_pages'
urlpatterns = [
    path('add/<slug:content_type_app_name>/<slug:content_type_model_name>/<int:parent_page_id>/', create.CreateView.as_view(), name='add'),
    path('add/<slug:content_type_app_name>/<slug:content_type_model_name>/<int:parent_page_id>/preview/', preview.PreviewOnCreate.as_view(), name='preview_on_add'),
    path('usage/<slug:content_type_app_name>/<slug:content_type_model_name>/', usage.content_type_use, name='type_use'),

    path('<int:page_id>/edit/', edit.EditView.as_view(), name='edit'),
    path('<int:page_id>/edit/preview/', preview.PreviewOnEdit.as_view(), name='preview_on_edit'),

    path('<int:page_id>/view_draft/', preview.view_draft, name='view_draft'),
    path('<int:parent_page_id>/add_subpage/', create.add_subpage, name='add_subpage'),
    path('<int:page_id>/delete/', delete.delete, name='delete'),
    path('<int:page_id>/unpublish/', unpublish.unpublish, name='unpublish'),
    path('<int:page_id>/convert_alias/', convert_alias.convert_alias, name='convert_alias'),

    path('search/', search.search, name='search'),

    path('<int:page_to_move_id>/move/', move.move_choose_destination, name='move'),
    path('<int:page_to_move_id>/move/<int:viewed_page_id>/', move.move_choose_destination, name='move_choose_destination'),
    path('<int:page_to_move_id>/move/<int:destination_id>/confirm/', move.move_confirm, name='move_confirm'),
    path('<int:page_to_move_id>/set_position/', ordering.set_page_position, name='set_page_position'),

    path('<int:page_id>/copy/', copy.copy, name='copy'),

    path('workflow/action/<int:page_id>/<slug:action_name>/<int:task_state_id>/', workflow.WorkflowAction.as_view(), name='workflow_action'),
    path('workflow/collect_action_data/<int:page_id>/<slug:action_name>/<int:task_state_id>/', workflow.CollectWorkflowActionData.as_view(), name='collect_workflow_action_data'),
    path('workflow/confirm_cancellation/<int:page_id>/', workflow.confirm_workflow_cancellation, name='confirm_workflow_cancellation'),
    path('workflow/preview/<int:page_id>/<int:task_id>/', workflow.preview_revision_for_task, name='workflow_preview'),
    path('workflow/status/<int:page_id>/', workflow.workflow_status, name='workflow_status'),

    path('moderation/<int:revision_id>/approve/', moderation.approve_moderation, name='approve_moderation'),
    path('moderation/<int:revision_id>/reject/', moderation.reject_moderation, name='reject_moderation'),
    path('moderation/<int:revision_id>/preview/', moderation.preview_for_moderation, name='preview_for_moderation'),

    path('<int:page_id>/privacy/', page_privacy.set_privacy, name='set_privacy'),

    path('<int:page_id>/lock/', lock.lock, name='lock'),
    path('<int:page_id>/unlock/', lock.unlock, name='unlock'),

    path('<int:page_id>/revisions/', revisions.revisions_index, name='revisions_index'),
    path('<int:page_id>/revisions/<int:revision_id>/view/', revisions.revisions_view, name='revisions_view'),
    path('<int:page_id>/revisions/<int:revision_id>/revert/', revisions.revisions_revert, name='revisions_revert'),
    path('<int:page_id>/revisions/<int:revision_id>/unschedule/', revisions.revisions_unschedule, name='revisions_unschedule'),
    re_path(r'^(\d+)/revisions/compare/(live|earliest|\d+)\.\.\.(live|latest|\d+)/$', revisions.revisions_compare, name='revisions_compare'),

    path('<int:page_id>/workflow_history/', history.workflow_history, name='workflow_history'),
    path('<int:page_id>/workflow_history/detail/<int:workflow_state_id>/', history.workflow_history_detail, name='workflow_history_detail'),

    path('<int:page_id>/history/', history.PageHistoryView.as_view(), name='history'),
]



"""An alternative urlconf module where Wagtail does not serve front-end URLs"""

from django.urls import include, path

from wagtail.admin import urls as wagtailadmin_urls
from wagtail.contrib.documents import urls as wagtaildocs_urls
from wagtail.contrib.images import urls as wagtailimages_urls


urlpatterns = [
    path('admin/', include(wagtailadmin_urls)),
    path('documents/', include(wagtaildocs_urls)),
    path('images/', include(wagtailimages_urls)),
]



# Generated by Django 3.1.8 on 2021-10-01 12:44

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import wagtail.models.admin


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wagtailcore', '0066_collection_management_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('submitted_notifications', models.BooleanField(default=True, help_text='Receive notification when a page is submitted for moderation', verbose_name='submitted notifications')),
                ('approved_notifications', models.BooleanField(default=True, help_text='Receive notification when your page edit is approved', verbose_name='approved notifications')),
                ('rejected_notifications', models.BooleanField(default=True, help_text='Receive notification when your page edit is rejected', verbose_name='rejected notifications')),
                ('updated_comments_notifications', models.BooleanField(default=True, help_text='Receive notification when comments have been created, resolved, or deleted on a page that you have subscribed to receive comment notifications on', verbose_name='updated comments notifications')),
                ('preferred_language', models.CharField(default='', help_text='Select language for the admin', max_length=10, verbose_name='preferred language')),
                ('current_time_zone', models.CharField(default='', help_text='Select your current time zone', max_length=40, verbose_name='current time zone')),
                ('avatar', models.ImageField(blank=True, upload_to=wagtail.models.admin.upload_avatar_to, verbose_name='profile picture')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='wagtail_userprofile', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'user profile',
                'verbose_name_plural': 'user profiles',
            },
        ),
    ]



# Generated by Django 2.2.10 on 2020-07-13 10:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0054_initial_locale'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='translation_key',
            field=models.UUIDField(editable=False, null=True),
        ),
        migrations.AddField(
            model_name='page',
            name='locale',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='wagtailcore.Locale'),
        ),
        migrations.AlterUniqueTogether(
            name='page',
            unique_together={('translation_key', 'locale')},
        ),
    ]



from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.test import TestCase

from wagtail.admin.filters import FilteredModelChoiceField
from wagtail.test.utils import WagtailTestUtils


User = get_user_model()


class TestFilteredModelChoiceField(TestCase, WagtailTestUtils):
    def setUp(self):
        self.musicians = Group.objects.create(name="Musicians")
        self.actors = Group.objects.create(name="Actors")

        self.david = self.create_user(
            'david', 'david@example.com', 'kn1ghtr1der', first_name="David", last_name="Hasselhoff"
        )
        self.david.groups.set([self.musicians, self.actors])

        self.kevin = self.create_user(
            'kevin', 'kevin@example.com', '6degrees', first_name="Kevin", last_name="Bacon"
        )
        self.kevin.groups.set([self.actors])

        self.morten = self.create_user(
            'morten', 'morten@example.com', 't4ke0nm3', first_name="Morten", last_name="Harket"
        )
        self.morten.groups.set([self.musicians])

    def test_with_relation(self):

        class UserForm(forms.Form):
            users = FilteredModelChoiceField(
                queryset=User.objects.order_by(User.USERNAME_FIELD), filter_field='id_group', filter_accessor='groups'
            )

        form = UserForm()
        html = str(form['users'])
        expected_html = """
            <select name="users" data-widget="filtered-select" data-filter-field="id_group" required id="id_users">
                <option value="" selected>---------</option>
                <option value="%(david)s" data-filter-value="%(musicians)s,%(actors)s">%(david_username)s</option>
                <option value="%(kevin)s" data-filter-value="%(actors)s">%(kevin_username)s</option>
                <option value="%(morten)s" data-filter-value="%(musicians)s">%(morten_username)s</option>
            </select>
        """ % {
            'david': self.david.pk, 'kevin': self.kevin.pk, 'morten': self.morten.pk,
            'musicians': self.musicians.pk, 'actors': self.actors.pk,
            'david_username': self.david.get_username(), 'kevin_username': self.kevin.get_username(),
            'morten_username': self.morten.get_username(),
        }
        self.assertHTMLEqual(html, expected_html)

    def test_with_callable(self):

        class UserForm(forms.Form):
            users = FilteredModelChoiceField(
                queryset=User.objects.order_by(User.USERNAME_FIELD), filter_field='id_group',
                filter_accessor=lambda user: user.groups.all()
            )

        form = UserForm()
        html = str(form['users'])
        expected_html = """
            <select name="users" data-widget="filtered-select" data-filter-field="id_group" required id="id_users">
                <option value="" selected>---------</option>
                <option value="%(david)s" data-filter-value="%(musicians)s,%(actors)s">%(david_username)s</option>
                <option value="%(kevin)s" data-filter-value="%(actors)s">%(kevin_username)s</option>
                <option value="%(morten)s" data-filter-value="%(musicians)s">%(morten_username)s</option>
            </select>
        """ % {
            'david': self.david.pk, 'kevin': self.kevin.pk, 'morten': self.morten.pk,
            'musicians': self.musicians.pk, 'actors': self.actors.pk,
            'david_username': self.david.get_username(), 'kevin_username': self.kevin.get_username(),
            'morten_username': self.morten.get_username(),
        }
        self.assertHTMLEqual(html, expected_html)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class WagtailLocalesAppConfig(AppConfig):
    name = 'wagtail.locales'
    label = 'wagtaillocales'
    verbose_name = _("Wagtail locales")



from django.urls import path

from wagtail.contrib.documents.views import chooser, documents, multiple


app_name = 'wagtaildocs'
urlpatterns = [
    path('', documents.IndexView.as_view(), name='index'),
    path('results/', documents.ListingResultsView.as_view(), name='listing_results'),
    path('add/', documents.add, name='add'),
    path('edit/<int:document_id>/', documents.edit, name='edit'),
    path('delete/<int:document_id>/', documents.delete, name='delete'),

    path('multiple/add/', multiple.AddView.as_view(), name='add_multiple'),
    path('multiple/<int:doc_id>/', multiple.EditView.as_view(), name='edit_multiple'),
    path('multiple/create_from_uploaded_document/<int:uploaded_document_id>/', multiple.CreateFromUploadedDocumentView.as_view(), name='create_multiple_from_uploaded_document'),
    path('multiple/<int:doc_id>/delete/', multiple.DeleteView.as_view(), name='delete_multiple'),
    path('multiple/delete_upload/<int:uploaded_document_id>/', multiple.DeleteUploadView.as_view(), name='delete_upload_multiple'),

    path('chooser/', chooser.ChooseView.as_view(), name='chooser'),
    path('chooser/results/', chooser.ChooseResultsView.as_view(), name='chooser_results'),
    path('chooser/<int:document_id>/', chooser.document_chosen, name='document_chosen'),
    path('chooser/upload/', chooser.chooser_upload, name='chooser_upload'),
    path('usage/<int:document_id>/', documents.usage, name='document_usage'),
]
